#!/usr/bin/env python
#
# Name: pyRX
# Desc: library for controlling rxaudio (www.xaudio.com)
# Date: 1/04/2001
# Vers: 0.1.0
#
# Copyright (C) 2001 Ben Wilson
#  
#
# This library is free software; you #can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
# 
#	
#	http://mpy3.thelocust.org
#	http://mpy3.sourceforge.net
#	Contact: ben@thelocust.org / thelocust@users.sourceforge.net
#
#
# init the player like so:
# 
#	foo = pyrx.player()
# 	foo.play("filename")
#	while 1:
#		if foo.state == "play":
#			foo.handle_audio()
#
#	You MUST handle the output of this thing.  Otherwise, it will eventually
#	stop taking commands.
#
#	The state that foo.state can be in are as follows:
#	play - it's playing a file
#	stop - it is stopped
#	pause - these are pretty self-explanatory, eh?
#	eof - when it finds the end of the file, it will stop automatically.
#		so, if you wanted that loop up above to play the same song
#		over and over again, you could add
#
#		elif foo.state == "eof":
#			foo.play("filename") 
#
#	
#	pyrx also keeps track of the following timestamps/etc.
#
#	tothour, totmin, and totsec - total time in file, broken down into hours,
#			minutes and seconds
#
#	duration - 	number of seconds TOTAL in file.  this is what tothour,totmin
#			and totsec are parsed from.  so, for a 6 minute file, this would be
#			360.
#	
#			therefore, tothour:totmin:totsec in a file that is 1 hour, 32 minutes
#			and 59 seconds long would be as such: 01:32:59
#
#	hour, min, and sec - current time stamps of playing file.  these are 
#			where rxaudio is in the current file.  
#	
#	progess - number of seconds into the file that rxaudio is currently playing
#
#	percent -  	silly text string that states progress/duration + "%", so if
#			so if duration was 150, and progress was 50, percent would look like
#			"33%"	
#
#	remain - remaining time in seconds
#	hourremain - remaining number of hours
#	minremain - remaining number of minutes
#	secremain - remaining number of seconds
#
#	level =
#	layer = MPEG layer type (MP3 is mpeg layer 3)
#	bitrate = most MP3s are at 128kbps, making this 128.  Usually, it's 128, 160,192, etc
#	frequency = sampling frequency.  usually 44100
#	mode = MONO, STEREO, JOINT-STEREO, etc.



import os,sys,select,re
import string
import FCNTL, fcntl
from fcntl import fcntl
from FCNTL import F_SETFL, F_SETFD, O_NONBLOCK
import time
import termios

class player:
	
	# states include stop, play, pause, eof
	state = ""

	debug_on = 0
	running = 0
	newinfo = 0

	def __init__(self):
		self.state = "stop"
		self.initvars()
		self.proc_start()

	def debug(self, text):
		if (self.debug_on == 1):
			print "pyRX: " + text + "\r"
		

	def proc_start(self):
		# shocking as it may seem, this starts the rxaudio process.  rxaudio must be in yer path,
		# but you knew that, right?
		self.debug("Starting Process")

		# opens the process -- sets the "process in" pipe as self.pin, and out as self.pout
		self.process = os.popen2("rxaudio","b",1)
		self.pin = self.process[0]
		self.pout = self.process[1]

		# does some initial setup for rxaudio
		r,w,e=select.select([self.pout],[self.pin],[])
		if w:
			self.pin.write("output_open\012set_player_mode 4\012set_output_port line\012")
			self.pin.flush()

	def shutdown(self):
		self.debug("Shutdown")
		self.running = 0

	def initvars(self):
		# clear all the vars
		self.duration = 0
		self.progress = 0
		self.remain = 0
		self.newinfo = 1

		self.percent = '0%'
		self.hour, self.min, self.sec = "00","00","00"
		self.tothour, self.totmin, self.totsec = "00","00","00"	
		self.hourremain, self.minremain, self.secremain = "00", "00", "00"
		self.frequency, self.bitrate = "",""

	def play(self, filename):
		# play a file
		
		self.initvars()

		self.debug("PLAY " + filename)
		
		self.state = "play"

		# stop the player, then flush to send the command
		if self.state == "play":
			self.pin.write("stop\012")
			self.pin.flush()


		# as long as the file is present, then open the file, and start to play it
		if (len(filename) > 0):
			self.pin.write("input_open " + filename + "\012play\012")
			
		#between commenting this and the timeout value of 0.1 in handle_audio (see below)
		#considerable speedups and timer accuracy can be had
		#self.pin.write("input_position_range 1\012")
		        self.pin.flush()

		
	def pause(self):
		# a pause toggle
                self.debug("PAUSE")
		
		if (self.state != "pause"):
			self.pin.write("pause\012")
			self.pin.flush()
			self.state = "pause"
		elif (self.state == "pause"):
			self.pin.write("play\012")
			self.pin.flush()
			self.state = "play"

	def stop(self):

                self.debug("STOP")
		
		if self.state != "stop":
			self.pin.write("stop\012")
			self.pin.flush()
		self.state = "stop"
		self.initvars()
				
	def handle_audio(self):
		#originally, i had a timeout of "1" in the fourth parameter of select.select there
		#that is the timeout value for getting whatever is on the line
		#1 was a little too much, and caused a general slowdown.  0.1 is much better -- much more
		# accurate, too.  0 is a non-blocking value.  we want it to block, though -- otherwise it may not catch
		# the output immediately.


		self.debug("Starting Loop!")
		self.running = 1

		# while we are running
		while self.running == 1:

			# hit the "process out" to see if there is something we need to pick up
			r,w,e=select.select([self.pout],[],[],0)
			# if there is
			if r:
				# get the line off the process out pipe
				x = self.pout.readline()

				# use a little regular expression magic to get the information from the line

				# stream info information
				y=re.match('MSG\snotify\sstream\sinfo\s\[level=(\d+),\slayer=(\d+),\sbitrate=(\d+),\sfrequency=(\d+),\smode=(\D+)\]',x)			
				if y:
					self.debug("NOTIFY INFO")
					self.level = str(y.group(1))
					self.layer = str(y.group(2))
					self.bitrate = str(y.group(3))
					self.frequency = str(y.group(4))
					self.mode = str(y.group(5))
					self.newinfo = 1
					

				# duration/file lenght info
				y=re.match('MSG\snotify\sduration\s\[(\d+)\]',x)
				if y:
					self.debug("NOTIFY DURATION")
					self.duration = int(y.group(1))
					self.tothour = string.zfill(str(self.duration/3600),2)
					self.totmin = string.zfill(str(self.duration%3600/60),2)
					self.totsec = string.zfill(str(self.duration%3600%60),2) 
					self.hourremain = self.tothour
					self.minremain = self.totmin
					self.secremain = self.totsec
					self.newinfo = 1

				# current time information
				y=re.match('MSG\snotify\stimecode[\s]\[(\d+):(\d+):(\d+):\d+\]',x)
				if y:
					self.debug("NOTIFY TIMECODE")

					self.hour = str(y.group(1)) 
					self.min = str(y.group(2))
					self.sec = str(y.group(3))

					self.progress = (int(y.group(1)*3600) + int(y.group(2))*60 + int(y.group(3)))

					self.remain = int(self.duration) - int(self.progress)
					self.hourremain = string.zfill(str(self.remain/3600),2)
					self.minremain = string.zfill(str(self.remain%3600/60),2)
					self.secremain = string.zfill(str(self.remain%3600%60),2)

					try:
						percent = 100*(float(self.progress)/self.duration)
						self.percent = repr(int(percent)) + "%"
					except ZeroDivisionError:
						pass
						#print "Zero Divide: " + str(self.progress) + "/" + str(self.duration)

					self.newinfo = 1

				# notify us of the end of a song.
				y=re.match('MSG\snotify\splayer\sstate\s\[EOF\]',x)
				if y:
					self.state = "eof"
					self.newinfo = 1



